# Generated by Django 3.1.3 on 2021-03-13 10:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('DjangoExampleApplication', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='privateattachment',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type'),
        ),
        migrations.AlterField(
            model_name='privateattachment',
            name='object_id',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name="Related Object's ID"),
        ),
        migrations.AlterField(
            model_name='publicattachment',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type'),
        ),
        migrations.AlterField(
            model_name='publicattachment',
            name='object_id',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name="Related Object's ID"),
        ),
    ]
