# Generated by Django 3.1.3 on 2020-11-15 20:23

import DjangoExampleApplication.models
from django.db import migrations, models
import django.db.models.deletion
import django_minio_backend.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('image', models.ImageField(storage=django_minio_backend.models.MinioBackend(bucket_name='django-backend-dev-public'), upload_to=django_minio_backend.models.iso_date_prefix)),
            ],
        ),
        migrations.CreateModel(
            name='PublicAttachment',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='Public Attachment ID')),
                ('object_id', models.PositiveIntegerField(verbose_name="Related Object's ID")),
                ('file', models.FileField(storage=django_minio_backend.models.MinioBackend(bucket_name='django-backend-dev-public'), upload_to=django_minio_backend.models.iso_date_prefix, verbose_name='Object Upload')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type')),
            ],
        ),
        migrations.CreateModel(
            name='PrivateAttachment',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='Public Attachment ID')),
                ('object_id', models.PositiveIntegerField(verbose_name="Related Object's ID")),
                ('file', models.FileField(storage=django_minio_backend.models.MinioBackend(bucket_name='django-backend-dev-private'), upload_to=DjangoExampleApplication.models.PrivateAttachment.set_file_path_name, verbose_name='Object Upload')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content Type')),
            ],
        ),
    ]
